#ifndef _PORTS_H_
#define _PORTS_H_

#include <avr/io.h>

// Note that these macros just serve your
// convenience, because there is ONE place in the
// source that says what port is connected to OE, e.g.
// You need not to hold definitions for PORTx, PINx and DDRx
// or alwas write the (1 << PADx) stuff.
//
// The optinizing compiler will deflate the cluttered C code
// that the macros expand to to the best asemmbler code it can.
// In general, this will be one machine instruction.
//
// However, the argument of these macros must be a compile time
// constants. Otherwise, your code will explode. I did not
// introduce checks by means of __builtin_constant_p
// or the like because I know what I am doing ;-)
#include <avr-port-macros.h>
#include <avr-port-enum.h>

// the very port assignments
enum
{
    // EPROMs
    PORT_OE  = PORTB_0,
 
    // 74*595
    PORT_SCK = PORTB_1,
    PORT_RCK = PORTC_4,
    PORT_SER = PORTC_5,
 
    // Taster 
    PORT_TAST0 = PORTB_2, // S1
    PORT_TAST1 = PORTC_2, // S2
    PORT_TAST2 = PORTC_0, // S3
 
    // VFD
    PORT_VFD_ID = PORT_ID (PORTD),
 
    PORT_RESET = PORTC_3,
    PORT_RS    = PORTB_3, // MOSI
    PORT_RW    = PORTC_1,
    PORT_E     = PORTB_5, // SCK
 
    // Lautsprecher
    PORT_SPEAK = PORTB_4, // MISO
 
    PORT_NIL
};

/*****************************************************************/

#endif /* _PORTS_H_ */
