#ifndef _VFD_H_
#define _VFD_H_

#include <avr/pgmspace.h>

#include "ports.h"

#define VFD_LINE1_START   0x40
#define VFD_VISIBLE_WIDTH 16
#define VFD_LINE_SIZE     40

//#define VFD_INTERFACE_4BIT
#define VFD_INTERFACE_8BIT_PARALLEL

#if !defined(VFD_INTERFACE_4BIT) && !defined(VFD_INTERFACE_8BIT_PARALLEL)
#error Define either 'VFD_INTERFACE_4BIT' or 'VFD_INTERFACE_8BIT_PARALLEL'
#endif
#if defined(VFD_INTERFACE_4BIT) && defined(VFD_INTERFACE_8BIT_PARALLEL)
#error Define either 'VFD_INTERFACE_4BIT' or 'VFD_INTERFACE_8BIT_PARALLEL'
#endif

// Wait until the VFD is ready by polling the BUSY flag
extern void vfd_wait_busy ();

// Send a byte to the VFD with RS line set to 0 (command)
extern void vfd_send0 (uint8_t);

// Send a byte to the VFD with RS line set to 1 (data)
extern void vfd_send1 (uint8_t);

// receive 1 byte from VFD
extern uint8_t vfd_receive ();

// Reset VFD by pulling down its RESET line for 10ms
// Write special characters to VFD's character generator RAM
extern void vfd_reset ();

// Set VFD with desired settings for blink mode, brightness, etc.
// Clear the VFD and set cursor home
extern void vfd_init ();

// Set Curser to position
// Top visible:  0..15
// Top hidden:  16..39
// Bottom visible: 0x40   ..0x40+15
// Bottom hidden:  0x40+16..0x40+39
extern void vfd_set_cursor (uint8_t);

// Put one character to VFD
extern void vfd_putc (char);

typedef struct
{
    uint8_t bright; // 0..4: 0=display off, 1-4=brighness > 0
 
    uint8_t cursor_blink;
    uint8_t cursor_on;
 
    uint8_t shift_mode;   // 0=shift cursor, 1=shift display
    uint8_t address_mode; // 0=address decrement, 1=adddress increment
} vfd_t;

extern vfd_t vfd;

#define VFD_DISPLAY_CLEAR  0
#define VFD_CURSOR_HOME    1
#define VFD_ENTRY_MODE_SET 2
#define VFD_ON_OFF_CTRL    3
#define VFD_SHFIT_SELECT   4
#define VFD_FUNCTION_SET   5
#define VFD_CG_RAM         6
#define VFD_DD_RAM         7

#define VFD_SET_DATA8 ((1 << VFD_FUNCTION_SET) | 0x10)
#define VFD_SET_DATA4  (1 << VFD_FUNCTION_SET)

#define VFD_BUSY 7

////////////////////////////////////////////////////////////

#ifdef VFD_INTERFACE_4BIT

#define VFD_OUT_DB                              \
do {                                            \
     MAKE_OUT (PORT_DB0);                       \
     MAKE_OUT (PORT_DB1);                       \
     MAKE_OUT (PORT_DB2);                       \
     MAKE_OUT (PORT_DB3);                       \
} while (0)

#define VFD_IN_DB                               \
do {                                            \
     MAKE_IN (PORT_DB0);                        \
     MAKE_IN (PORT_DB1);                        \
     MAKE_IN (PORT_DB2);                        \
     MAKE_IN (PORT_DB3);                        \
} while (0)

#define VFD_CLR_DB                              \
do {                                            \
     CLR (PORT_DB0);                            \
     CLR (PORT_DB1);                            \
     CLR (PORT_DB2);                            \
     CLR (PORT_DB3);                            \
} while (0)

#endif /* VFD_INTERFACE_4BIT */

////////////////////////////////////////////////////////////

#ifdef VFD_INTERFACE_8BIT_PARALLEL

#define VFD_OUT_DB                              \
do {                                            \
    N_TO_DDR (PORT_VFD_ID) = 0xff;              \
} while (0)

#define VFD_IN_DB                               \
do {                                            \
    N_TO_DDR (PORT_VFD_ID) = 0;                 \
} while (0)

#define VFD_CLR_DB                              \
do {                                            \
    N_TO_PORT (PORT_VFD_ID) = 0;                \
} while (0)

#endif /* VFD_INTERFACE_8BIT_PARALLEL */
#endif /* _VFD_H_ */
